using System.Collections.Generic;
using System.ComponentModel;
using Oxide.Core.Plugins;
using UnityEngine;
using System;

/// <summary>
/// Edited by klauz24
/// </summary>

namespace Oxide.Plugins
{
    [Info("Death Messages", "Wil Simpson & Mr. Blue", 1.3)]
    [Description("Custom death messages for your server")]
    class DeathMessages : HurtworldPlugin
    {
        [PluginReference("KillCounter")]
        Plugin KillCounter;

        void OnServerInitialized()
        {
            GameManager.Instance.ServerConfig.ChatDeathMessagesEnabled = false;
        }

        protected override void LoadDefaultMessages()
        {
            // English
            lang.RegisterMessages(new Dictionary<string, string>
            {
                {"DMessages_Prefix","<color=red>[Death Messages]</color> "},
                {"DMessages_Unknown","{Victim} just died on a mystic way"},
                {"DMessages_MessageWithKillCounter","{Victim} got killed by {Killer} [{killerKills}] ({distance}m)"},
                {"DMessages_MessageWithoutKillCounter","{Victim} got killed by {Killer} ({distance}m)"},
                {"EntityStats/Sources/Fall Damage","{Victim} just died"},
                {"EntityStats/Sources/Damage Over Time","{Victim} just died"},
                {"EntityStats/Sources/Radiation Poisoning","{Victim} just died"},
                {"EntityStats/Sources/Radiation","{Victim} just died"},
                {"EntityStats/Sources/Starvation","{Victim} just died"},
                {"EntityStats/BinaryEffects/Starving","{Victim} just died"},
                {"EntityStats/BinaryEffects/Burning","{Victim} just died"},
                {"Too Cold","{Victim} just died"},
                {"EntityStats/BinaryEffects/Hyperthermia","{Victim} just died"},
                {"EntityStats/Sources/Hypothermia","{Victim} just died"},
                {"EntityStats/Sources/Hyperthermia","{Victim} just died"},
                {"EntityStats/Sources/Asphyxiation","{Victim} just died"},
                {"EntityStats/Sources/Poison","{Victim} just died"},
                {"EntityStats/Sources/Burning","{Victim} just died"},
                {"EntityStats/BinaryEffects/Hypothermia","{Victim} just died"},
                {"EntityStats/Sources/Suicide","{Victim} just died"},
                {"EntityStats/Sources/Explosives","{Victim} just died"},
                {"EntityStats/Sources/a Vehicle Impact","{Victim} just died"},
                {"Creatures/Tokar","{Victim} got killed by Tokar"},
                {"Creatures/Shigi","{Victim} got killed by a Shigi"},
                {"Creatures/Bor","{Victim} got killed by a Bor"},
                {"Creatures/Radiation Bor","{Victim} got killed by a Radiation Bor"},
                {"Creatures/Yeti","{Victim} got killed by a Yeti"},
                {"Creatures/Sasquatch","{Victim} got killed by a Sasquatch"},
                {"Machines/Medusa Vine", "{Victim} got killed by Medusa Trap"},
                {"Machines/Landmine", "{Victim} got killed by Landmine"},
            }, this, "en");

            // Polish
            lang.RegisterMessages(new Dictionary<string, string>
            {
                {"DMessages_Prefix","<color=red>[Death Messages]</color> "},
                {"DMessages_Unknown","{Victim} po prostu umarl"},
                {"DMessages_MessageWithKillCounter","{Victim} został zabity przez {Killer} [{killerKills}] ({distance}m)"},
                {"DMessages_MessageWithoutKillCounter","{Victim} został zabity przez {Killer} ({distance}m)"},
                {"EntityStats/Sources/Fall Damage","{Victim} Nie przetrwał upadku"},
                {"EntityStats/Sources/Damage Over Time","{Victim} po prostu umarl"},
                {"EntityStats/Sources/Radiation Poisoning","{Victim} zginął przez napromieniowanie"},
                {"EntityStats/Sources/Radiation","{Victim} zginął przez wygłodzenie"},
                {"EntityStats/Sources/Starvation","{Victim} zginął przez wygłodzenie"},
                {"EntityStats/BinaryEffects/Starving","{Victim} zginął przez wygłodzenie"},
                {"EntityStats/BinaryEffects/Burning","{Victim} just died"},
                {"Too Cold","{Victim} zginął przez zimno"},
                {"EntityStats/BinaryEffects/Hyperthermia","{Victim} zginął przez przegrzanie"},
                {"EntityStats/Sources/Hypothermia","{Victim} zginął przez zimno"},
                {"EntityStats/Sources/Hyperthermia","{Victim} zginął przez przegrzanie"},
                {"EntityStats/Sources/Asphyxiation","{Victim} po prostu umarl"},
                {"EntityStats/Sources/Poison","{Victim} umarł przez trucizne"},
                {"EntityStats/Sources/Burning","{Victim} spalił się"},
                {"EntityStats/BinaryEffects/Hypothermia","{Victim} zginął przez zimno"},
                {"EntityStats/Sources/Suicide","{Victim} popełnił samobójstwo"},
                {"EntityStats/Sources/Explosives","{Victim} wyleciał w powietrze"},
                {"EntityStats/Sources/a Vehicle Impact","{Victim} został rozjechany"},
                {"Creatures/Tokar","{Victim} zabity przez Tokar"},
                {"Creatures/Shigi","{Victim} zabity przez Shigi"},
                {"Creatures/Bor","{Victim} zabity przez Dzika"},
                {"Creatures/Radiation Bor","{Victim} zabity przez Napromieniowanego Dzika"},
                {"Creatures/Yeti","{Victim} zabity przez Yeti"},
                {"Creatures/Sasquatch","{Victim} zabity przez Sasquatch"},
                {"Machines/Medusa Vine", "{Victim} zabity przez Medusa"},
                {"Machines/Landmine", "{Victim} został wysadzony przez mine"},
            }, this, "pl");

        }

        void OnPlayerDeath(PlayerSession playerSession, EntityEffectSourceData dataSource)
        {
            if (playerSession == null || dataSource == null)
            {
                return;
            }
            else
            {
                string VictimName = playerSession.Identity.Name;
                string SDKey = !string.IsNullOrEmpty(dataSource.SourceDescriptionKey) ? dataSource.SourceDescriptionKey : Singleton<GameManager>.Instance.GetDescriptionKey(dataSource.EntitySource);
                if (SDKey.EndsWith("(P)"))
                {
                    string killerName = SDKey.Substring(0, SDKey.Length - 3);
                    var name = GetPlayerSession(VictimName);
                    var killer_Session = GetPlayerSession(killerName);
                    var distance = Mathf.Round(Vector3.Distance(playerSession.WorldPlayerEntity.transform.position, killer_Session.WorldPlayerEntity.transform.position)).ToString();
                    if (KillCounter != null)
                    {
                        var killerkills = KillCounter.Call("AddKill", playerSession, dataSource);
                        Server.Broadcast(GetLang("DMessages_Prefix") + GetLang("DMessages_MessageWithKillCounter").Replace("{Killer}", killerName.ToString()).Replace("{Victim}", VictimName.ToString()).Replace("{distance}", distance).Replace("{killerKills}", killerkills.ToString()));
                    }
                    else
                    {
                        Server.Broadcast(GetLang("DMessages_Prefix") + GetLang("DMessages_MessageWithoutKillCounter").Replace("{Killer}", killerName.ToString()).Replace("{Victim}", VictimName.ToString()).Replace("{distance}", distance));
                    }
                }
                else
                {
                    if (lang.GetMessage(SDKey, this) != SDKey)
                    {
                        Server.Broadcast(GetLang("DMessages_Prefix") + GetLang(SDKey, this).Replace("{Victim}", VictimName.ToString()));
                    }
                    else
                    {
                        Server.Broadcast(GetLang("DMessages_Prefix") + GetLang("Unknown", this).Replace("{Victim}", Name.ToString()));
                        throw new WarningException("This death type was not found: " + SDKey);
                    }
                }
            }
        }

        string GetNameOfObject(UnityEngine.GameObject obj)
        {
            var ManagerInstance = GameManager.Instance;
            return ManagerInstance.GetDescriptionKey(obj);
        }

        bool IsValidSession(PlayerSession session)
        {
            return session != null && session?.SteamId != null && session.IsLoaded && session.Identity.Name != null && session.Identity != null &&
                   session.WorldPlayerEntity?.transform?.position != null;
        }

        PlayerSession GetPlayerSession(string identifier)
        {
            foreach (PlayerIdentity identity in GameManager.Instance.GetIdentifierMap().Values)
            {
                PlayerSession session = identity.ConnectedSession;
                if (IsValidSession(session) && session.Identity.Name.ToLower().Equals(identifier.ToLower()))
                {
                    return session;
                }
            }
            return null;
        }

        string GetLang(string key, object SteamId = null) => lang.GetMessage(key, this, SteamId == null ? null : SteamId.ToString());
    }
}
